package j3d.examples.boids;
// WrapFlocking3D.java
// Andrew Davison, April 2005, ad@fivedots.coe.psu.ac.th
// Sirinart Sakarin, March 2003, s4210315@calvin.coe.psu.ac.th

/* The flocking predators and prey are created and added to
   the scene in addFlockingBoids(). Obstacles for the boids are
   also created there.

   The rest of the code is standard. 
*/

import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;

import javax.media.j3d.*;
import javax.swing.*;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import java.awt.*;

import j3d.examples.checkers.CheckerFloor;

// Holds the 3D canvas where the loaded image is displayed

public class WrapFlocking3D {
    private static final int PWIDTH = 512;   // size of panel
    private static final int PHEIGHT = 512;
    private static final int BOUNDSIZE = 100;  // larger than world

    private static final Point3d USERPOSN = new Point3d(0, 5, 20);
    // initial user position


    private SimpleUniverse su;
    private BranchGroup sceneBG;
    private BoundingSphere bounds;   // for environment nodes
    private final JPanel boidPanel = getBirdPanel();

    private JPanel getBirdPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new BorderLayout());
        jp.setOpaque(false);
        jp.setPreferredSize(new Dimension(PWIDTH, PHEIGHT));
        return jp;
    }

    public WrapFlocking3D(int numPreds, int numPrey, int numObs) {


        GraphicsConfiguration config =
                SimpleUniverse.getPreferredConfiguration();
        Canvas3D canvas3D = new Canvas3D(config);
        boidPanel.add("Center", canvas3D);
        canvas3D.setFocusable(true);     // give focus to the canvas
        canvas3D.requestFocus();

        su = new SimpleUniverse(canvas3D);
        createSceneGraph(numPreds, numPrey, numObs);
        initUserPosition();        // set user's viewpoint
        orbitControls(canvas3D);   // controls for moving the viewpoint

        su.addBranchGraph(sceneBG);
    }


    private void createSceneGraph(int numPreds, int numPrey, int numObs) {
        sceneBG = new BranchGroup();
        bounds = new BoundingSphere(new Point3d(0, 0, 0), BOUNDSIZE);
        lightScene();         // add the lights
        addBlueSkyBackground();      // add the sky
        sceneBG.addChild(new CheckerFloor().getBG());  // add the floor

        addFlockingBoids(numPreds, numPrey, numObs);
    }


    private void lightScene()
        /* One ambient light, 2 directional lights */ {
        Color3f white = new Color3f(1.0f, 1.0f, 1.0f);

        // Set up the ambient light
        AmbientLight ambientLightNode = new AmbientLight(white);
        ambientLightNode.setInfluencingBounds(bounds);
        sceneBG.addChild(ambientLightNode);

        // Set up the directional lights
        Vector3f light1Direction = new Vector3f(-1.0f, -1.0f, -1.0f);
        // left, down, backwards
        Vector3f light2Direction = new Vector3f(1.0f, -1.0f, 1.0f);
        // right, down, forwards

        DirectionalLight light1 =
                new DirectionalLight(white, light1Direction);
        light1.setInfluencingBounds(bounds);
        sceneBG.addChild(light1);

        DirectionalLight light2 =
                new DirectionalLight(white, light2Direction);
        light2.setInfluencingBounds(bounds);
        sceneBG.addChild(light2);
    }  // end of lightScene()


    private void addBlueSkyBackground() {
        Background back = new Background();
        back.setApplicationBounds(bounds);
        back.setColor(0.17f, 0.65f, 0.92f);    // sky colour
        sceneBG.addChild(back);
    }

    /* OrbitBehaviour allows the
    user to rotate around the scene, and to
zoom in and out. */
    private void orbitControls(Canvas3D c) {
        OrbitBehavior orbit =
                new OrbitBehavior(c, OrbitBehavior.REVERSE_ALL);
        orbit.setSchedulingBounds(bounds);

        ViewingPlatform vp = su.getViewingPlatform();
        vp.setViewPlatformBehavior(orbit);
    }  // end of orbitControls()

    // Set the user's initial viewpoint using lookAt()
    private void initUserPosition() {
        ViewingPlatform vp = su.getViewingPlatform();
        TransformGroup steerTG = vp.getViewPlatformTransform();

        Transform3D t3d = new Transform3D();
        steerTG.getTransform(t3d);

        // args are: viewer posn, where looking, up direction
        t3d.lookAt(USERPOSN, new Point3d(0, 0, 0), new Vector3d(0, 1, 0));
        t3d.invert();

        steerTG.setTransform(t3d);
    }  // end of initUserPosition()

    private void addFlockingBoids(int numPreds, int numPrey, int numObs) {
        // create obstacles
        j3d.examples.boids.Obstacles obs = new Obstacles(numObs);
        sceneBG.addChild(obs.getObsBG());  // add obstacles to the scene

        // make the predator manager
        PredatorBehavior predBeh = new PredatorBehavior(numPreds, obs);
        predBeh.setSchedulingBounds(bounds);
        sceneBG.addChild(predBeh.getBoidsBG());  // add predators to the scene

        // make the prey manager
        PreyBehavior preyBeh = new j3d.examples.boids.PreyBehavior(numPrey, obs);
        preyBeh.setSchedulingBounds(bounds);
        sceneBG.addChild(preyBeh.getBoidsBG());  // add prey to the scene

        // tell behaviours about each other
        predBeh.setPreyBeh(preyBeh);
        preyBeh.setPredBeh(predBeh);

    }


    public JPanel getBoidPanel() {
        return boidPanel;
    }
}